;Windows XP Patch Inspector by HeidiR v2.2
;Srcipt reads the registry to get a list of installed Windows XP patches and writes the patch numbers (KB*) to a sorted text file.
;The script also optionally searches your Windows source patch folder and writes available uninstalled patch numbers (KB*) to a sorted text file.
;Author: HeidiR
;2/6/10 Initial release
;2/7/10 Added the optional search for available uninstalled patch numbers (KB*) and wrtites them to a sorted text file.
;2/9/10 Added installed patch count to text file
;2/12/10 Added patch installation process
;=== Main-Start ===
;To configure for your machine:
;	Open the windows_xp_patch_inspector.ini file
;	Change: winxp_patch_path. This key is the folder path on your machine where all available Windows XP patch .exe files are located.
;				If you choose not to check for uninstalled patches (next key), this key is not used and can be ignored.
;	Confirm: check_if_uninstalled_winxp_patches_exist=1
;				Options: 1 - Search your winxp_patch_path folder (above) for patches that you have in the folder but not yet installed.
;							0 - DO NOT search your winxp_patch_path folder for patches that you have in the folder but not yet installed.
;	Confirm: display_installed_winxp_patches_on_completion=1
;				Options: 1 - Open the installed_windows_xp_patch_list.txt in your default editor on completion
;							0 - DO NOT open the installed_windows_xp_patch_list.txt in your default editor on completion
;	Confirm: display_available_winxp_patches_on_completion=1
;				Options: 1 - Open the available_windows_xp_patch_list.txt in your default editor on completion
;							0 - DO NOT open the available_windows_xp_patch_list.txt in your default editor on completion
;	Confirm: install_available_winxp_patches=1
;				Options: 1 - Install ALL available patches in available_windows_xp_patch_list.txt (Will be prompted to confirm at runtime)
;							0 - DO NOT Install ALL available patches
;
;INIT
	#include<array_sort.au3>

	Global $KB_INSTALLED_TXT = ""
	Global $KB_AVAILABLE_TXT = ""
	Global $AR[1] ; KB array for sorting
	Global $INI = @ScriptDir & "\windows_xp_patch_inspector.ini"
	Global $INSTALLED_PATCH_LOG = @ScriptDir & "\installed_windows_xp_patch_list.txt"
	Global $AVAILABLE_PATCH_LOG = @ScriptDir & "\available_windows_xp_patch_list.txt"
	Global $INSTALL_PATCH_LOG = @ScriptDir & "\winxp patch install log.txt"
	Global $FH_INSTALL_LOG, $FH_AVAILABLE_LOG, $PATCH_PROCESSED_CNT=0, $PATCH_INSTALL_CNT=0
	Global $PATCH_SOURCE_FOLDER_PATH
	Global $CHECK_IF_UNINSTALLED_WINXP_PATCHES_EXIST
	Global $DISPLAY_INSTALLED_WINXP_PATCHES_ON_COMPLETION
	Global $DISPLAY_AVAILABLE_WINXP_PATCHES_ON_COMPLETION
	Global $INSTALL_AVAILABLE_WINXP_PATCHES

	get_ini()

;PROCESS
	Msgbox(64, "Windows XP Patch Inspector by HeidiR", "Starting Patch Processing", 2)

	;Look for installed patches
	check_installed_patches()

	;Show installed patch text file in notepad
	If $DISPLAY_INSTALLED_WINXP_PATCHES_ON_COMPLETION == 1 Then
		ShellExecute($INSTALLED_PATCH_LOG, "", "", "", @SW_MINIMIZE)
	Endif

	;Check for uninstalled patches available in source folder
	If $CHECK_IF_UNINSTALLED_WINXP_PATCHES_EXIST == 1 Then
		check_patches_exist()
		;Show available patch text file in notepad
		If $DISPLAY_AVAILABLE_WINXP_PATCHES_ON_COMPLETION == 1 Then
			ShellExecute($AVAILABLE_PATCH_LOG, "", "", "", @SW_MINIMIZE)
		Endif
	Endif

	;Restore window
	If $DISPLAY_INSTALLED_WINXP_PATCHES_ON_COMPLETION == 1 Then
		WinSetState("installed_windows_xp_patch_list.txt", "", @SW_RESTORE)
		Sleep(500)
		WinActivate("installed_windows_xp_patch_list.txt")
	Endif

	;Reposition window
	If $DISPLAY_AVAILABLE_WINXP_PATCHES_ON_COMPLETION == 1 AND $CHECK_IF_UNINSTALLED_WINXP_PATCHES_EXIST == 1 Then
		WinSetState("available_windows_xp_patch_list.txt", "", @SW_RESTORE)
		Sleep(1000)
		Local $winpos_ar = WinGetPos("available_windows_xp_patch_list.txt")
		If @error == 0 Then
			WinActivate("available_windows_xp_patch_list.txt")
			WinWaitActive("available_windows_xp_patch_list.txt")
			WinMove("available_windows_xp_patch_list.txt", "", $winpos_ar[0] + 100, $winpos_ar[1] + 40)
		Endif
	Endif

	;Install available patches
	If $INSTALL_AVAILABLE_WINXP_PATCHES == 1 Then
		If FileGetSize($AVAILABLE_PATCH_LOG) == 0 Then
			Msgbox(64, "Windows XP Patch Inspector by HeidiR", "No Available Patches to be Installed Exist in Source Folder.")
		Else
			$install_yn = Msgbox(33, "Windows XP Patch Inspector by HeidiR", "Select OK to Install ALL available patches" & @LF & "Select Cancel to Cancel the Install" & @LF & @LF & "Edit " & $AVAILABLE_PATCH_LOG & @LF & "NOW to remove any patch numbers you do NOT want to install before selecting OK.")
			If $install_yn == 1 Then
				install_patches()
			Endif
		Endif
	Endif

;TERMINATE
	Sleep(500)
	Msgbox(64, "Windows XP Patch Inspector by HeidiR", "Patch Processing Complete", 3)

	Exit


;=========================================================

Func check_installed_patches()
	;read the registry to get a list of iinstalled patches
	For $i = 1 to 1000
	    $key1= RegEnumKey("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Updates\Windows XP", $i)
	    If @error <> 0 then ExitLoop

	    savekb2array($key1)

	  	For $j = 1 to 1000
	        $key2 = RegEnumKey("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Updates\Windows XP" & "\" & $key1, $j)
		    If @error <> 0 then ExitLoop

		    savekb2array($key2)
	    Next
	Next

	;Sort installed patch numbers
	_ArraySort($AR)

	;Write installed patch log file
	write_kb_log_file($INSTALLED_PATCH_LOG, "I")

Endfunc


;Check for uninstalled patches available in source folder
Func check_patches_exist()
	ReDim $AR[1]
	Local $hdl = FileFindFirstFile($PATCH_SOURCE_FOLDER_PATH & "\*.exe")
	Local $filename

	If $hdl == -1 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Patch File Source Folder Empty: " & @LF & $PATCH_SOURCE_FOLDER_PATH)
		exit
	Endif

	While 1
	    $filename = FileFindNextFile($hdl)
	    If @error Then ExitLoop
		;Files
		If StringInStr(FileGetAttrib($PATCH_SOURCE_FOLDER_PATH & "\" & $filename),"D") == 0 Then
			;get kb number from file name
			$pos = StringInStr($filename, "KB")
			If $pos > 0 Then
				$kb = StringMid ($filename, $pos, 8)
				;check to see if patch file has been installed
				If StringInStr($KB_INSTALLED_TXT, $kb) == 0 Then
					Msgbox(64, "Windows XP Patch Inspector by HeidiR - Warning", "Uninstalled Patch File Exists for: " & @LF & $kb, 1)
					savekb2array($kb)
				Endif
			Else
				Msgbox(48, "Windows XP Patch Inspector by HeidiR - Warning", "Non-Patch File Exists in source folder. " & @LF & $filename, 3)
			Endif
		Endif

	Wend

	If Ubound($AR) > 1 Then
		;Sort available patch numbers
		_ArraySort($AR)
	Endif

	;Write installed patch log file
	write_kb_log_file($AVAILABLE_PATCH_LOG, "A")

Endfunc

;Write KB patch numbers to a text file
Func write_kb_log_file($log, $type)
	Local $rc
	Local $fh = FileOpen($log, 2) ; 2-Write w/file overwrite

	; Check if file opened for writing
	If $fh == -1 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Unable to open log file: " & $log)
	    Return
	EndIf

	Local $ar_cnt = Ubound($AR)

	;Create text string from sorted array to write to file and write file
	Switch $type
	        Case "I" ;Installed
				For $i = 1 to $ar_cnt -1
					$KB_INSTALLED_TXT &= $AR[$i] & @CRLF
				Next
				$KB_INSTALLED_TXT &= @CRLF & "Installed Patch Count: " & $ar_cnt -1
				$rc = FileWrite($fh, $KB_INSTALLED_TXT)

	        Case "A"
				For $i = 1 to $ar_cnt -1
					$KB_AVAILABLE_TXT &= $AR[$i] & @CRLF
				Next
				$rc = FileWrite($fh, $KB_AVAILABLE_TXT)

	Endswitch

	;Write file return code
	If $rc == 0 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Unable to write log file: " & $log)
	EndIf

	FileClose($fh)

Endfunc

;Save KB numbers to an array for sorting before writing to a file
Func savekb2array ($txt)
	Local $ar_cnt = Ubound($AR)

	;Save only keys that begin with KB
    $pos = StringInStr($txt, "KB", 1)

    If $pos > 0 Then
    	$kb = StringMid ($txt, $pos, 8)
		ReDim $AR[$ar_cnt +1]
		$AR[$ar_cnt] = $kb
	Endif

Endfunc


Func get_ini()

	If FileExists ($INI) Then
		$PATCH_SOURCE_FOLDER_PATH = IniRead($INI, "windows_xp_patch_inspector", "winxp_patch_path", "")
		$CHECK_IF_UNINSTALLED_WINXP_PATCHES_EXIST = IniRead($INI, "windows_xp_patch_inspector", "check_if_uninstalled_winxp_patches_exist", "0")
		$DISPLAY_INSTALLED_WINXP_PATCHES_ON_COMPLETION = IniRead($INI, "windows_xp_patch_inspector", "display_installed_winxp_patches_on_completion", "1")
		$DISPLAY_AVAILABLE_WINXP_PATCHES_ON_COMPLETION = IniRead($INI, "windows_xp_patch_inspector", "display_available_winxp_patches_on_completion", "0")
		$INSTALL_AVAILABLE_WINXP_PATCHES = IniRead($INI, "windows_xp_patch_inspector", "install_available_winxp_patches", "0")
	Else
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Ini file does not exist")
		exit
	Endif

Endfunc


;Install patches main driver
Func install_patches()

	;Open available patch log for reading
	$FH_AVAILABLE_LOG = open_log_file($AVAILABLE_PATCH_LOG, 0)

	;Open patch install log for writing
	$FH_INSTALL_LOG = open_log_file($INSTALL_PATCH_LOG, 2)

	write_log_header()

	;Install all available patches
	While 1
	    $kb = FileReadLine($FH_AVAILABLE_LOG)
	    If @error = -1 OR StringLen($kb) <> 8 Then ExitLoop ;eof
	    $PATCH_PROCESSED_CNT += 1
	    MsgBox(0, "Windows XP Patch Inspector by HeidiR", "Processing: " & $kb, 1)
	    install_one_patch($kb)

	Wend

	write_log_footer()

	close_log_files()

	If $PATCH_PROCESSED_CNT == 0 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Available patch log file is empty: " & $AVAILABLE_PATCH_LOG)
	Else
		Msgbox(64, "Windows XP Patch Inspector by HeidiR", "Patch Installation Completed. Log file will be displayed." & @LF & @LF & "Patches Processed: " & $PATCH_PROCESSED_CNT & @LF & "Patches Installed:   " & $PATCH_INSTALL_CNT, 3)
		ShellExecute($INSTALL_PATCH_LOG, "", @SW_MAXIMIZE)
	Endif

Endfunc


;Write header info to patch install log
Func write_log_header()
	write_install_log_file("Windows XP Patch Inspector by HeidiR - Patch Installation Log." & @CRLF & @CRLF)
	write_install_log_file("Start: " & @MON & "/" & @MDAY & "/" & @YEAR & " " & @HOUR & ":" & @MIN & ":" & @SEC & @CRLF)
	write_install_log_file("List of patch files excuted successfully:" & @CRLF)

Endfunc


;Write footer info to patch install log
Func write_log_footer()
	write_install_log_file(@CRLF & "End: " & @MON & "/" & @MDAY & "/" & @YEAR & " " & @HOUR & ":" & @MIN & ":" & @SEC & @CRLF)
	write_install_log_file("Total Patches Installed: " & $PATCH_INSTALL_CNT)

Endfunc


Func open_log_file($fn, $type)
	$fh = FileOpen($fn, $type) ;type: 2-Write w/file overwrite, 0-read

	; Check if file opened for writing
	If $fh == -1 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Unable to open file: " & $fn)
		close_log_files()
	    Exit
	EndIf

	Return $fh

Endfunc


Func write_install_log_file($str)
	Local $rc = FileWrite($FH_INSTALL_LOG, $str & @CRLF)

	If $rc == 0 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Unable to write log file: " & $INSTALL_PATCH_LOG)
	EndIf

Endfunc


Func close_log_files()

	FileClose($FH_AVAILABLE_LOG)
	FileClose($FH_INSTALL_LOG)

Endfunc


Func install_one_patch($kb)

	$patch_file = get_patch_file_name($kb)

	If StringLen($patch_file) == 0 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Patch File Source File Not Found for Patch Number: " & $kb & @CRLF & "Patch Install Skipped.")
		Return
	Endif

;	Msgbox(0, "Source File", $patch_file, 1)

	Run($PATCH_SOURCE_FOLDER_PATH & $patch_file)

	automate_ms_install_wizard()

	$PATCH_INSTALL_CNT += 1
	write_install_log_file($kb)

Endfunc


;Find patch file that matches passed in kb number in source folder
Func get_patch_file_name($kb)
	Local $hdl = FileFindFirstFile($PATCH_SOURCE_FOLDER_PATH & "\*.exe")
	Local $filename

	If $hdl == -1 Then
		Msgbox(16, "Windows XP Patch Inspector by HeidiR - Error", "Patch File Source Folder Empty: " & @LF & $PATCH_SOURCE_FOLDER_PATH)
		exit
	Endif

	While 1
	    $filename = FileFindNextFile($hdl)
	    If @error Then ExitLoop

		;Files
		If StringInStr(FileGetAttrib($PATCH_SOURCE_FOLDER_PATH & "\" & $filename),"D") == 0 Then
			;get kb number from file name
			$pos = StringInStr($filename, $kb)
			If $pos > 0 Then
				Return $filename ;Patch file name
			Endif
		Endif

	Wend

	Return "" ;no patch file found

Endfunc


Func automate_ms_install_wizard()
	    ;Initial dialog
	    WinWaitActive ("Software Update Installation Wizard")
		Sleep(500)
	    If WinGetText ( "Software Update Installation Wizard", "install the following software update:") Then
		Send("!N") ;Alt N
		Sleep(500)
	    Endif

	    ;License dialog (Does not always appear)
	    WinWaitActive ("Software Update Installation Wizard")
	    Sleep(500)
	    If WinGetText ( "Software Update Installation Wizard", "license agreement") Then
		Send("!A") ;Alt A
		Sleep(200)
		Send("!N") ;Alt N
		Sleep(500)
	    Endif

	    While 1
	        WinWaitActive ("Software Update Installation Wizard")
	        Sleep(500)
			If WinGetText ( "Software Update Installation Wizard", "click Finish.") Then
				ExitLoop
			Endif
			Sleep(500)
	    WEnd

	    WinActivate ("Software Update Installation Wizard")
	    Sleep(500)

	    If WinGetText ( "Software Update Installation Wizard", "restart Windows") Then
			Send("!D") ;Alt D (no restart)
			Send("{TAB}") ;Alt Tab
			Sleep(500)
			Send("{ENTER}") ;(Finish)
		Else
			Sleep(500)
			Send("{ENTER}") ;(Finish)
	    Endif

	    Sleep(5000)

Endfunc